///File : MainPanel.cpp
//------------------------------------------------------------------------------
/*
    This file is part of massmol.

    massmol is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    massmol is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with massmol.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

//------------------------------------------------------------------------------
#include <wx/wxhtml.h>
#include <wx/stdpaths.h>
//------------------------------------------------------------------------------
#include "MasseMolaire.hpp"
#include "wxMain.hpp"
#include "MainPanel.hpp"
#include "DialogAbout.hpp"
#include "findFile.hpp"
//------------------------------------------------------------------------------
BEGIN_EVENT_TABLE(MainPanel, wxPanel)
    EVT_BUTTON(wxID_EXIT,  MainPanel::OnQuit)
    EVT_BUTTON(wxID_ABOUT, MainPanel::OnAPropos)
    EVT_TEXT(Id_TxtMoleculeEnter, MainPanel::OnCalcul)
END_EVENT_TABLE()

//------------------------------------------------------------------------------
MainPanel::MainPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
{
    m_MaCMasseMolaire.OpenAtomesFile(findFile(_T("data/masseMolaireAtomique")));
    //SetBackgroundColour(*wxWHITE);

    wxBoxSizer *BtQuitterAndAPropos = new wxBoxSizer( wxHORIZONTAL );
    m_BtAPropos = new wxButton(this, wxID_ABOUT, _("&About"));
    BtQuitterAndAPropos->Add(m_BtAPropos, 0, wxRIGHT, 5);
    m_BtQuitter = new wxButton(this, wxID_EXIT, _("&Quit"));
    BtQuitterAndAPropos->Add(m_BtQuitter);

    wxBoxSizer *MainSizer = new wxBoxSizer( wxVERTICAL );
    m_Molecule = new wxTextCtrl(this, Id_TxtMoleculeEnter);
    MainSizer->Add(m_Molecule, 0, wxEXPAND | wxALL, 5);
    m_ResultMol = new wxHtmlWindow(this);
    MainSizer->Add(m_ResultMol, 1, wxEXPAND | wxLEFT | wxRIGHT, 5);
    MainSizer->Add(BtQuitterAndAPropos, 0, wxALIGN_RIGHT | wxALL, 5);

    SetSizer(MainSizer);

    m_ResultMol->SetFocus();
    m_Molecule->SetValue(_T("H2O"));
}
//------------------------------------------------------------------------------
void MainPanel::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    exit(EXIT_SUCCESS);
}
//------------------------------------------------------------------------------
void MainPanel::OnCalcul(wxCommandEvent& WXUNUSED(event))
{
    m_ResultMol->SetPage(m_MaCMasseMolaire.GetInfoMolecule(m_Molecule->GetValue()));
}
//------------------------------------------------------------------------------
void MainPanel::OnAPropos(wxCommandEvent& WXUNUSED(event))
{
    DialogAbout MyDialogAbout(this, wxID_ANY);
    MyDialogAbout.ShowModal();

}
//------------------------------------------------------------------------------
