///File : MasseMolaire.cpp
//------------------------------------------------------------------------------
/*
    This file is part of massmol.

    massmol is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    massmol is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with massmol.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/textfile.h>

//------------------------------------------------------------------------------

#include "MasseMolaire.hpp"

//------------------------------------------------------------------------------
CMasseMolaire::CMasseMolaire(const wxString &FileName)
{
    CMasseMolaire();

    OpenAtomesFile(FileName);
}

//------------------------------------------------------------------------------
CMasseMolaire::CMasseMolaire()
{
    isOk = false;
    m_nbAtomes = 0;
    m_TblNomAtomes = NULL;
    m_tblMasseMolAtomes = NULL;
    m_tblAtomeA = NULL;
    m_tblAtomeZ = NULL;

    m_MoleculeDecomposeeEnCours.AtomesNames = NULL;
    m_MoleculeDecomposeeEnCours.nbAtomes = NULL;
}

//------------------------------------------------------------------------------
CMasseMolaire::~CMasseMolaire()
{
    delete[] m_TblNomAtomes;
    delete[] m_tblMasseMolAtomes;
    delete[] m_tblAtomeA;
    delete[] m_tblAtomeZ;

    delete[] m_MoleculeDecomposeeEnCours.AtomesNames;
    delete[] m_MoleculeDecomposeeEnCours.nbAtomes;
}

//------------------------------------------------------------------------------
void CMasseMolaire::DecomposeMolecule(const wxString &MoleculeName)
{
    m_MoleculeDecomposeeEnCours.nomHTMLMolecule = _T("");
    m_MoleculeDecomposeeEnCours.isOk = false;
    m_MoleculeDecomposeeEnCours.nbAtomInMolecul = 0;
    delete[] m_MoleculeDecomposeeEnCours.AtomesNames;
    delete[] m_MoleculeDecomposeeEnCours.nbAtomes;
    m_MoleculeDecomposeeEnCours.AtomesNames = NULL;
    m_MoleculeDecomposeeEnCours.nbAtomes = NULL;

    if(!isOk)
    {
        return;
    }

    unsigned int pos = 0;
    wxArrayString nbAtomesStrTbl, AtomesNamesStrTbl;

    while(pos < MoleculeName.Len())
    {
        if(!IsMaj(MoleculeName[pos])) // Si la 1er lettre n'est pas une Maj
        {
            return;
        }

        else
        {
            if(IsMin(MoleculeName[pos + 1])) // Si la 2é lettre est minuscule
            {
                AtomesNamesStrTbl.Add(MoleculeName.Mid(pos, 2));
            }
            else
            {
                AtomesNamesStrTbl.Add(MoleculeName.Mid(pos, 1));
            }
            pos += AtomesNamesStrTbl.Last().Len(); //On avance du nb de lettre de la molécule...
        }

        if(GetMasseMolaireAtome(AtomesNamesStrTbl.Last()) == -1)
        {
            return;
        }

        if(IsMin(MoleculeName[pos]))
        {
            return;
        }
        else if(IsMaj(MoleculeName[pos]) || pos == (MoleculeName.Len())) // Si la 3é lettre est maj ou la molécule est fini==> Multiplieur = 1
        {
            wxString str;
            str << 1;
            nbAtomesStrTbl.Add(str);
        }
        else //Il y a un nb d'atome... mais sur combien de caractéres s'écrit-il ?
        {
            wxString tempNbStr;
            while(pos < MoleculeName.Len() && !IsMaj(MoleculeName[pos]))
            {
                tempNbStr << MoleculeName[pos];
                pos++;
            }
            nbAtomesStrTbl.Add(tempNbStr);
        }
    }

    if(nbAtomesStrTbl.Count() == AtomesNamesStrTbl.Count())
    {
        m_MoleculeDecomposeeEnCours.nbAtomes = new double[nbAtomesStrTbl.Count()];
        m_MoleculeDecomposeeEnCours.AtomesNames = new wxString[nbAtomesStrTbl.Count()];

        for(unsigned int i = 0; i < nbAtomesStrTbl.Count(); i++)
        {
            if(!nbAtomesStrTbl[i].ToDouble(&m_MoleculeDecomposeeEnCours.nbAtomes[i])) //Si on ne peut pas lire le nombre...
                return;
            m_MoleculeDecomposeeEnCours.AtomesNames[i] = AtomesNamesStrTbl[i];

            m_MoleculeDecomposeeEnCours.nomHTMLMolecule << m_MoleculeDecomposeeEnCours.AtomesNames[i]; //On ajoute chaque atome au nom de la molécule...
            if(m_MoleculeDecomposeeEnCours.nbAtomes[i] != 1)
            {
                m_MoleculeDecomposeeEnCours.nomHTMLMolecule << _T("<sub>") << m_MoleculeDecomposeeEnCours.nbAtomes[i] << _T("</sub>"); //On ajoute le nb d'atomes si il sont != de 1.
            }
        }
        m_MoleculeDecomposeeEnCours.nbAtomInMolecul = AtomesNamesStrTbl.Count();
        m_MoleculeDecomposeeEnCours.isOk = true;
        return;
    }
    else
    {
        return;
    }
}

//------------------------------------------------------------------------------
wxString CMasseMolaire::GetAZCalcul(const wxString &MoleculeName)
{
    wxString CalculL1, CalculAll;
    double NTotal = 0, ZTotal = 0;

    for(unsigned int i = 0; i < m_MoleculeDecomposeeEnCours.nbAtomInMolecul; i++)
    {
        CalculL1 << m_MoleculeDecomposeeEnCours.AtomesNames[i];
        CalculL1 << _T("<ul><li>Z=N(e<sup>-</sup>)=");
        CalculL1 << GetZAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i]);
        CalculL1 << _T("</li><li>N=");
        CalculL1 << GetNAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i]);
        CalculL1 << _T("</li><li>A=Z+N=");
        CalculL1 << GetNAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i]) + GetZAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i]);
        CalculL1 << _T("</li></ul><br /><br />");

        NTotal += GetNAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i]) * m_MoleculeDecomposeeEnCours.nbAtomes[i];
        ZTotal += GetZAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i]) * m_MoleculeDecomposeeEnCours.nbAtomes[i];
    }

    CalculL1 << m_MoleculeDecomposeeEnCours.nomHTMLMolecule;
    CalculL1 << _T("<ul><li>Z=N(e<sup>-</sup>)=");
    CalculL1 << ZTotal;
    CalculL1 << _T("</li><li>N=");
    CalculL1 << NTotal;
    CalculL1 << _T("</li><li>A=Z+N=");
    CalculL1 << ZTotal + NTotal;
    CalculL1 << _T("</li></ul>");

    // On rassemble tous
    CalculAll << _T("<p>") << CalculL1 << _T("</p>");

    return CalculAll;
}

//------------------------------------------------------------------------------
wxString CMasseMolaire::GetMasseAtomiqueCalcul(const wxString &MoleculeName)
{
    wxString CalculL1, CalculL2, CalculL3, CalculAll;
    double NTotal = 0;

    for(unsigned int i = 0; i < m_MoleculeDecomposeeEnCours.nbAtomInMolecul; i++)
    {
        CalculL1 << m_MoleculeDecomposeeEnCours.AtomesNames[i];
        CalculL1 << _T("<ul><li>m=N * m(p)</li><li>m=");
        CalculL1 << GetNAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i]) + GetZAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i]);
        CalculL1 << _T(" * 1.673 * 10<sup>-24</sup></li><li>m=");
        CalculL1 << double((GetNAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i])) + GetZAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i])) * 1.673;
        CalculL1 << _T(" * 10<sup>-24</sup> g</li></ul><br /><br />");

        NTotal += (GetNAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i]) + GetZAtom(m_MoleculeDecomposeeEnCours.AtomesNames[i])) * m_MoleculeDecomposeeEnCours.nbAtomes[i];
    }
    CalculL1 << m_MoleculeDecomposeeEnCours.nomHTMLMolecule;
    CalculL1 << _T("<ul><li>m=N * m(p)</li><li>m=");
    CalculL1 << NTotal;
    CalculL1 << _T(" * 1.673e-24</li><li>m=");
    //CalculL1 << _T(" * 1.673 * 10<sup>-24</sup></li><li>m=");
    CalculL1 << NTotal * 1.673e-24;
    CalculL1 << _T(" g</li></ul>");
    //CalculL1 << _T(" * 10<sup>-24</sup> g</li></ul>");

    // On rassemble tous
    CalculAll << _T("<p>");
    CalculAll << _("We consider that every nucleons have the same mass and the electrons mass doesn't matter.");
    CalculAll << _T("<br /><br />");
    CalculAll << CalculL1;
    CalculAll << _T("</p>");

    return CalculAll;
}

wxString CMasseMolaire::CanMoleculExist(const wxString &MoleculeName)
{
    return _T("");
}

//------------------------------------------------------------------------------
wxString CMasseMolaire::GetMasseMolaireCalcul(const wxString &MoleculeName)
{
    wxString CalculL1, CalculL2, CalculL3;

    double masse = 0;

    for(unsigned int i = 0; i < m_MoleculeDecomposeeEnCours.nbAtomInMolecul; i++)
    {
        double masseAtomiqueEnCours = GetMasseMolaireAtome(m_MoleculeDecomposeeEnCours.AtomesNames[i]);

        CalculL1 << _T(" ") << m_MoleculeDecomposeeEnCours.nbAtomes[i] << _T(" * M<sub>") << m_MoleculeDecomposeeEnCours.AtomesNames[i] << _T("</sub> +");
        CalculL2 << _T(" ") << m_MoleculeDecomposeeEnCours.nbAtomes[i] << _T(" * ") << masseAtomiqueEnCours << _T(" +");

        masse+= m_MoleculeDecomposeeEnCours.nbAtomes[i] * masseAtomiqueEnCours;
    }

    // On met le nom de la molécule
    CalculL1 = _T("M(") + m_MoleculeDecomposeeEnCours.nomHTMLMolecule + _T(") =") + CalculL1;
    CalculL2 = _T("M(") + m_MoleculeDecomposeeEnCours.nomHTMLMolecule + _T(") =") + CalculL2;
    CalculL3 = _T("M(") + m_MoleculeDecomposeeEnCours.nomHTMLMolecule + _T(") =") + CalculL3;

    // On jarte les " +" finaux;
    CalculL1 = CalculL1.Mid(0, CalculL1.Len() -2);
    CalculL2 = CalculL2.Mid(0, CalculL2.Len() -2);

    // On fait la ligne 3.
    CalculL3 << _T(" ") << masse << _T(" g.mol<sup>-1</sup>");

    // On rassemble tous
    wxString CalculAll;
    CalculAll << _T("<p>") << CalculL1 << _T("<br />") << CalculL2 << _T("<br />") << CalculL3 << _T("</p>");

    return CalculAll;
}

//------------------------------------------------------------------------------
wxString CMasseMolaire::GetInfoMolecule(const wxString &MoleculeName)
{
    if(MoleculeName == _T(""))
        return _("Error : Empty field.");

    if(MoleculeName == _T("plume powa"))
    {
        return _T("Miaou !");
    }

    if(!isOk)
        return _("Error : Can't open the atom information file.");

    DecomposeMolecule(MoleculeName);

    if(!m_MoleculeDecomposeeEnCours.isOk)
        return _("Error : bad molecule.");

    wxString All;

    All << _T("<html><body><h3>");
    All << _("Molecular mass");
    All << _T("</h3>");
    All << GetMasseMolaireCalcul(MoleculeName);
    All << _T("<h3>") << _("Number of protons, neutrons and electrons") << _T("</h3>");
    All << GetAZCalcul(MoleculeName);
    All << _T("<h3>") << _("Mass") << _T("</h3>");
    All << GetMasseAtomiqueCalcul(MoleculeName);
    All << _T("</body></html>");

    return All;
}
//------------------------------------------------------------------------------

bool CMasseMolaire::IsMin(char letter) const
{
    for(int i = 0; i < 25; i++)
    {
        if(letter == 'a' + i)
            return true;
    }
    return false;
}

//------------------------------------------------------------------------------
bool CMasseMolaire::IsMaj(char letter) const
{
    for(int i = 0; i < 25; i++)
    {
        if(letter == 'A' + i)
            return true;
    }
    return false;
}

//------------------------------------------------------------------------------
double CMasseMolaire::GetMasseMolaireAtome(const wxString &AtomName) const
{
    if(!isOk)
        return -1;

    for(int i = 0; i < m_nbAtomes; i++)
    {
        if(AtomName == m_TblNomAtomes[i])
            return m_tblMasseMolAtomes[i];
    }
    return -1;
}

//------------------------------------------------------------------------------
int CMasseMolaire::GetZAtom(const wxString &AtomName) const
{
    if(!isOk)
        return -1;

    for(int i = 0; i < m_nbAtomes; i++)
    {
        if(AtomName == m_TblNomAtomes[i])
            return m_tblAtomeZ[i];
    }
    return -1;
}

//------------------------------------------------------------------------------
int CMasseMolaire::GetNAtom(const wxString &AtomName) const
{
    if(!isOk)
        return -1;

    for(int i = 0; i < m_nbAtomes; i++)
    {
        if(AtomName == m_TblNomAtomes[i])
            return m_tblAtomeA[i];
    }
    return -1;
}

//------------------------------------------------------------------------------
void CMasseMolaire::OpenAtomesFile(const wxString &FileName)
{
    delete[] m_TblNomAtomes;
    delete[] m_tblMasseMolAtomes;
    delete[] m_tblAtomeA;
    delete[] m_tblAtomeZ;
    m_TblNomAtomes = NULL;
    m_tblMasseMolAtomes = NULL;
    m_tblAtomeA = NULL;
    m_tblAtomeZ = NULL;

    wxTextFile File(FileName);
    if(!File.Exists())
    {
        isOk = false;
        return;
    }

    if(!File.Open())
    {
        isOk = false;
        return;
    }

    m_nbAtomes = File.GetLineCount() / 3;
    m_TblNomAtomes = new wxString[m_nbAtomes];
    m_tblMasseMolAtomes = new double[m_nbAtomes];
    m_tblAtomeA = new int[m_nbAtomes];
    m_tblAtomeZ = new int[m_nbAtomes];

    wxString str = File.GetFirstLine();

    int i = 0;

    while(!File.Eof())
    {
        int posEgal = str.Find('=');

        if(posEgal != wxNOT_FOUND)
        {
            m_TblNomAtomes[i] = str.Mid(0, posEgal);
            // Si il y a des points pour les décimaux, ça ne marche pas. Il faut des virgules...
            wxString tempNb = str.Mid(posEgal + 1, str.Len() - 1);
            tempNb.Replace(wxString(_T('.')),wxString(_T(',')),true);
            tempNb.ToDouble(&m_tblMasseMolAtomes[i]);

            long temp;
            str = File.GetNextLine();
            str.ToLong(&temp);
            m_tblAtomeZ[i] = temp;

            str = File.GetNextLine();
            str.ToLong(&temp);
            m_tblAtomeA[i] = temp;

            i++;
        }
        str = File.GetNextLine();
    }
    isOk = true;
    m_nbAtomes = i;
}
