///File : wxMain.cpp
//------------------------------------------------------------------------------
/*
    This file is part of massmol.

    massmol is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    massmol is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with massmol.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

//------------------------------------------------------------------------------
#include <wx/intl.h>
#include <wx/stdpaths.h>
//------------------------------------------------------------------------------
#include "MainPanel.hpp"
#include "wxMain.hpp"
#include "findFile.hpp"
//------------------------------------------------------------------------------
IMPLEMENT_APP(App)

#ifdef __WXMAC__
    #include <ApplicationServices/ApplicationServices.h>
#endif
//------------------------------------------------------------------------------
bool App::OnInit()
{
    #ifdef __WXMAC__
    ProcessSerialNumber PSN;
    GetCurrentProcess(&PSN);
    TransformProcessType(&PSN,kProcessTransformToForegroundApplication);
    #endif

    // Création du Handler pour les images (permet de les ouvrir)...
    wxInitAllImageHandlers();

    // Gestion de la langue
    // Ajout des préfixes possibles de chemins d'accès aux catalogues
    //wxLocale::AddCatalogLookupPathPrefix(wxT("."));
    //wxLocale::AddCatalogLookupPathPrefix(wxT(".."));
    //wxLocale::AddCatalogLookupPathPrefix(_T("locale"));
    wxLocale::AddCatalogLookupPathPrefix(findFile(_T("locale")));
    // Mise en place de la langue par défaut du système
    m_locale.Init(wxLANGUAGE_DEFAULT);
    {
       wxLogNull noLog; // Supprime les erreurs si les catalogues n'existent pas
       // Catalogue de l'application
       m_locale.AddCatalog(_T("prog"));
       // Catalogue de wxWidgets
       m_locale.AddCatalog(_T("wxstd"));
    }

    MainFrame *frame = new MainFrame(_("Molar molecular mass"),  wxDefaultPosition, wxSize(500, 450), wxTAB_TRAVERSAL | wxMINIMIZE_BOX | wxMAXIMIZE_BOX | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN);
    frame->Show(true);

    return true;
}
//------------------------------------------------------------------------------
MainFrame::MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size, long style) : wxFrame(NULL, -1, title, pos, size, style)
{
    #if defined(__WXMSW__)
        SetIcon(wxIcon(_T("monicone")));
    #elif defined(__UNIX__)
        wxString ImgFileName = findFile(_T("data/icon.png"));
        if(wxFileExists(ImgFileName))
        {
            SetIcon(wxIcon(ImgFileName, wxBITMAP_TYPE_PNG));
        }
    #endif //#ifdef WIN32
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));

	wxBoxSizer* Sizer1;
	Sizer1 = new wxBoxSizer( wxVERTICAL );

	m_panel1 = new MainPanel( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
	Sizer1->Add( m_panel1, 1, wxEXPAND);

	this->SetSizer( Sizer1 );
	//this->Layout();
}
